@@ -36,6 +36,15 @@ command -v $CMAKE || (echo "CMake could not be found...aborting!"; exit 1)
 cd libmongocrypt
 mkdir cmake-build
 cd cmake-build
+
+for suffix in "dll" "dylib" "so"; do
+    if test -f "mongo_csfle_v1.$suffix"; then
+        ADDITIONAL_CMAKE_FLAGS="$ADDITIONAL_CMAKE_FLAGS -DMONGOCRYPT_TESTING_CSFLE_FILE=$PWD/mongo_csfle_v1.$suffix"
+    fi
+done
+
+ADDITIONAL_CMAKE_FLAGS="$ADDITIONAL_CMAKE_FLAGS -DENABLE_MORE_WARNINGS_AS_ERRORS=ON"
+
 $CMAKE -DCMAKE_BUILD_TYPE=RelWithDebInfo $ADDITIONAL_CMAKE_FLAGS "${LIBMONGOCRYPT_EXTRA_CMAKE_FLAGS}" -DCMAKE_C_FLAGS="${LIBMONGOCRYPT_EXTRA_CFLAGS}" -DCMAKE_PREFIX_PATH="${BSON_INSTALL_PREFIX}" "-DCMAKE_INSTALL_PREFIX=${MONGOCRYPT_INSTALL_PREFIX}" -DCMAKE_EXPORT_COMPILE_COMMANDS=ON ../
 
 if [ "$CONFIGURE_ONLY" ]; then
@@ -48,9 +57,21 @@ $CMAKE --build . --target install --config RelWithDebInfo
 # library to support consumers that static link to libmongocrypt
 find ${BSON_INSTALL_PREFIX} \( -name libbson-static-1.0.a -o -name bson-1.0.lib -o -name bson-static-1.0.lib \) -execdir cp {} $(dirname $(find ${MONGOCRYPT_INSTALL_PREFIX} -name libmongocrypt-static.a -o -name mongocrypt-static.lib)) \;
 $CMAKE --build . --target test-mongocrypt --config RelWithDebInfo
-$CMAKE --build ./kms-message --target test_kms_request --config RelWithDebInfo
+$CMAKE --build . --target test_kms_request --config RelWithDebInfo
 cd $evergreen_root
 
+# MONGOCRYPT-372, ensure macOS universal builds contain both x86_64 and arm64 architectures.
+if [ "$MACOS_UNIVERSAL" = "ON" ]; then
+    echo "Checking if libmongocrypt.dylib contains both x86_64 and arm64 architectures..."
+    ARCHS=$(lipo -archs $MONGOCRYPT_INSTALL_PREFIX/lib/libmongocrypt.dylib)
+    if [[ "$ARCHS" == *"x86_64"* && "$ARCHS" == *"arm64"* ]]; then
+        echo "Checking if libmongocrypt.dylib contains both x86_64 and arm64 architectures... OK"
+    else
+        echo "Checking if libmongocrypt.dylib contains both x86_64 and arm64 architectures... ERROR. Got: $ARCHS"
+        exit
+    fi
+fi
+
 if [ "$PPA_BUILD_ONLY" ]; then
     echo "Only building/installing for PPA";
     exit 0;
