@@ -60,6 +60,7 @@ import static com.mongodb.crypt.capi.CAPI.mongocrypt_setopt_kms_provider_local;
 import static com.mongodb.crypt.capi.CAPI.mongocrypt_setopt_kms_providers;
 import static com.mongodb.crypt.capi.CAPI.mongocrypt_setopt_log_handler;
 import static com.mongodb.crypt.capi.CAPI.mongocrypt_setopt_schema_map;
+import static com.mongodb.crypt.capi.CAPI.mongocrypt_setopt_use_need_kms_credentials_state;
 import static com.mongodb.crypt.capi.CAPI.mongocrypt_status;
 import static com.mongodb.crypt.capi.CAPI.mongocrypt_status_destroy;
 import static com.mongodb.crypt.capi.CAPI.mongocrypt_status_new;
@@ -151,6 +152,10 @@ class MongoCryptImpl implements MongoCrypt {
             }
         }
 
+        if (options.isNeedsKmsCredentialsStateEnabled()) {
+            mongocrypt_setopt_use_need_kms_credentials_state(wrapped);
+        }
+
         if (options.getKmsProviderOptions() != null) {
             try (BinaryHolder binaryHolder = toBinary(options.getKmsProviderOptions())) {
                 success = mongocrypt_setopt_kms_providers(wrapped, binaryHolder.getBinary());
