@@ -1,7 +1,11 @@
 'use strict';
 
-module.exports = function(modules) {
+module.exports = function (modules) {
   const tls = require('tls');
+  const net = require('net');
+  const fs = require('fs');
+  const { once } = require('events');
+  const { SocksClient } = require('socks');
 
   // Try first to import 4.x name, fallback to 3.x name
   const MongoNetworkTimeoutError =
@@ -12,13 +16,14 @@ module.exports = function(modules) {
   const databaseNamespace = common.databaseNamespace;
   const collectionNamespace = common.collectionNamespace;
   const MongoCryptError = common.MongoCryptError;
-  const BufferList = require('bl');
+  const { BufferPool } = require('./buffer_pool');
 
   // libmongocrypt states
   const MONGOCRYPT_CTX_ERROR = 0;
   const MONGOCRYPT_CTX_NEED_MONGO_COLLINFO = 1;
   const MONGOCRYPT_CTX_NEED_MONGO_MARKINGS = 2;
   const MONGOCRYPT_CTX_NEED_MONGO_KEYS = 3;
+  const MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS = 7;
   const MONGOCRYPT_CTX_NEED_KMS = 4;
   const MONGOCRYPT_CTX_READY = 5;
   const MONGOCRYPT_CTX_DONE = 6;
@@ -30,11 +35,20 @@ module.exports = function(modules) {
     [MONGOCRYPT_CTX_NEED_MONGO_COLLINFO, 'MONGOCRYPT_CTX_NEED_MONGO_COLLINFO'],
     [MONGOCRYPT_CTX_NEED_MONGO_MARKINGS, 'MONGOCRYPT_CTX_NEED_MONGO_MARKINGS'],
     [MONGOCRYPT_CTX_NEED_MONGO_KEYS, 'MONGOCRYPT_CTX_NEED_MONGO_KEYS'],
+    [MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS, 'MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS'],
     [MONGOCRYPT_CTX_NEED_KMS, 'MONGOCRYPT_CTX_NEED_KMS'],
     [MONGOCRYPT_CTX_READY, 'MONGOCRYPT_CTX_READY'],
     [MONGOCRYPT_CTX_DONE, 'MONGOCRYPT_CTX_DONE']
   ]);
 
+  const INSECURE_TLS_OPTIONS = [
+    'tlsInsecure',
+    'tlsAllowInvalidCertificates',
+    'tlsAllowInvalidHostnames',
+    'tlsDisableOCSPEndpointCheck',
+    'tlsDisableCertificateRevocationCheck'
+  ];
+
   /**
    * @ignore
    * @callback StateMachine~executeCallback
@@ -163,6 +177,22 @@ module.exports = function(modules) {
           return;
         }
 
+        case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS: {
+          autoEncrypter
+            .askForKMSCredentials()
+            .then(kmsProviders => {
+              context.provideKMSProviders(
+                !Buffer.isBuffer(kmsProviders) ? bson.serialize(kmsProviders) : kmsProviders
+              );
+              this.execute(autoEncrypter, context, callback);
+            })
+            .catch(err => {
+              callback(err, null);
+            });
+
+          return;
+        }
+
         case MONGOCRYPT_CTX_NEED_KMS: {
           const promises = [];
 
@@ -224,42 +254,138 @@ module.exports = function(modules) {
       const options = { host: parsedUrl[0], servername: parsedUrl[0], port };
       const message = request.message;
 
-      return new Promise((resolve, reject) => {
-        const buffer = new BufferList();
-        const socket = tls.connect(options, () => {
-          socket.write(message);
-        });
+      // TODO(NODE-3959): We can adopt `for-await on(socket, 'data')` with logic to control abort
+      // eslint-disable-next-line no-async-promise-executor
+      return new Promise(async (resolve, reject) => {
+        const buffer = new BufferPool();
 
-        socket.once('timeout', () => {
-          socket.removeAllListeners();
-          socket.destroy();
+        let socket;
+        let rawSocket;
+
+        function destroySockets() {
+          for (const sock of [socket, rawSocket]) {
+            if (sock) {
+              sock.removeAllListeners();
+              sock.destroy();
+            }
+          }
+        }
+
+        function ontimeout() {
+          destroySockets();
           reject(new MongoCryptError('KMS request timed out'));
-        });
-
-        socket.once('error', err => {
-          socket.removeAllListeners();
-          socket.destroy();
+        }
 
+        function onerror(err) {
+          destroySockets();
           const mcError = new MongoCryptError('KMS request failed');
           mcError.originalError = err;
           reject(mcError);
+        }
+
+        if (this.options.proxyOptions && this.options.proxyOptions.proxyHost) {
+          rawSocket = net.connect({
+            host: this.options.proxyOptions.proxyHost,
+            port: this.options.proxyOptions.proxyPort || 1080
+          });
+
+          rawSocket.on('timeout', ontimeout);
+          rawSocket.on('error', onerror);
+          try {
+            await once(rawSocket, 'connect');
+            options.socket = (
+              await SocksClient.createConnection({
+                existing_socket: rawSocket,
+                command: 'connect',
+                destination: { host: options.host, port: options.port },
+                proxy: {
+                  // host and port are ignored because we pass existing_socket
+                  host: 'iLoveJavaScript',
+                  port: 0,
+                  type: 5,
+                  userId: this.options.proxyOptions.proxyUsername,
+                  password: this.options.proxyOptions.proxyPassword
+                }
+              })
+            ).socket;
+          } catch (err) {
+            return onerror(err);
+          }
+        }
+
+        const tlsOptions = this.options.tlsOptions;
+        if (tlsOptions) {
+          const kmsProvider = request.kmsProvider;
+          const providerTlsOptions = tlsOptions[kmsProvider];
+          if (providerTlsOptions) {
+            const error = this.validateTlsOptions(kmsProvider, providerTlsOptions);
+            if (error) reject(error);
+            this.setTlsOptions(providerTlsOptions, options);
+          }
+        }
+        socket = tls.connect(options, () => {
+          socket.write(message);
         });
 
+        socket.once('timeout', ontimeout);
+        socket.once('error', onerror);
+
         socket.on('data', data => {
           buffer.append(data);
           while (request.bytesNeeded > 0 && buffer.length) {
             const bytesNeeded = Math.min(request.bytesNeeded, buffer.length);
-            request.addResponse(buffer.slice(0, bytesNeeded));
-            buffer.consume(bytesNeeded);
+            request.addResponse(buffer.read(bytesNeeded));
           }
 
           if (request.bytesNeeded <= 0) {
-            socket.end(resolve);
+            // There's no need for any more activity on this socket at this point.
+            destroySockets();
+            resolve();
           }
         });
       });
     }
 
+    /**
+     * @ignore
+     * Validates the provided TLS options are secure.
+     *
+     * @param {string} kmsProvider The KMS provider name.
+     * @param {ClientEncryptionTLSOptions} tlsOptions The client TLS options for the provider.
+     *
+     * @returns {Error} If any option is invalid.
+     */
+    validateTlsOptions(kmsProvider, tlsOptions) {
+      const tlsOptionNames = Object.keys(tlsOptions);
+      for (const option of INSECURE_TLS_OPTIONS) {
+        if (tlsOptionNames.includes(option)) {
+          return new MongoCryptError(
+            `Insecure TLS options prohibited for ${kmsProvider}: ${option}`
+          );
+        }
+      }
+    }
+
+    /**
+     * @ignore
+     * Sets only the valid secure TLS options.
+     *
+     * @param {ClientEncryptionTLSOptions} tlsOptions The client TLS options for the provider.
+     * @param {Object} options The existing connection options.
+     */
+    setTlsOptions(tlsOptions, options) {
+      if (tlsOptions.tlsCertificateKeyFile) {
+        const cert = fs.readFileSync(tlsOptions.tlsCertificateKeyFile);
+        options.cert = options.key = cert;
+      }
+      if (tlsOptions.tlsCAFile) {
+        options.ca = fs.readFileSync(tlsOptions.tlsCAFile);
+      }
+      if (tlsOptions.tlsCertificateKeyFilePassword) {
+        options.passphrase = tlsOptions.tlsCertificateKeyFilePassword;
+      }
+    }
+
     /**
      * @ignore
      * Fetches collection info for a provided namespace, when libmongocrypt
