@@ -33,20 +33,15 @@
 /* Crypto primitives. These either call the native built in crypto primitives or
  * user supplied hooks. */
 static bool
-_crypto_aes_256_cbc_encrypt (_mongocrypt_crypto_t *crypto,
-                             const _mongocrypt_buffer_t *enc_key,
-                             const _mongocrypt_buffer_t *iv,
-                             const _mongocrypt_buffer_t *in,
-                             _mongocrypt_buffer_t *out,
-                             uint32_t *bytes_written,
-                             mongocrypt_status_t *status)
+_crypto_aes_256_cbc_encrypt (_mongocrypt_crypto_t *crypto, aes_256_args_t args)
 {
-   if (enc_key->len != MONGOCRYPT_ENC_KEY_LEN) {
+   mongocrypt_status_t *status = args.status;
+   if (args.key->len != MONGOCRYPT_ENC_KEY_LEN) {
       CLIENT_ERR ("invalid encryption key length");
       return false;
    }
 
-   if (iv->len != MONGOCRYPT_IV_LEN) {
+   if (args.iv->len != MONGOCRYPT_IV_LEN) {
       CLIENT_ERR ("invalid iv length");
       return false;
    }
@@ -55,35 +50,29 @@ _crypto_aes_256_cbc_encrypt (_mongocrypt_crypto_t *crypto,
       mongocrypt_binary_t enc_key_bin, iv_bin, out_bin, in_bin;
       bool ret;
 
-      _mongocrypt_buffer_to_binary (enc_key, &enc_key_bin);
-      _mongocrypt_buffer_to_binary (iv, &iv_bin);
-      _mongocrypt_buffer_to_binary (out, &out_bin);
-      _mongocrypt_buffer_to_binary (in, &in_bin);
+      _mongocrypt_buffer_to_binary (args.key, &enc_key_bin);
+      _mongocrypt_buffer_to_binary (args.iv, &iv_bin);
+      _mongocrypt_buffer_to_binary (args.out, &out_bin);
+      _mongocrypt_buffer_to_binary (args.in, &in_bin);
 
       ret = crypto->aes_256_cbc_encrypt (crypto->ctx,
                                          &enc_key_bin,
                                          &iv_bin,
                                          &in_bin,
                                          &out_bin,
-                                         bytes_written,
+                                         args.bytes_written,
                                          status);
       return ret;
    }
-   return _native_crypto_aes_256_cbc_encrypt (
-      enc_key, iv, in, out, bytes_written, status);
+   return _native_crypto_aes_256_cbc_encrypt (args);
 }
 
 
 static bool
-_crypto_aes_256_cbc_decrypt (_mongocrypt_crypto_t *crypto,
-                             const _mongocrypt_buffer_t *iv,
-                             const _mongocrypt_buffer_t *enc_key,
-                             const _mongocrypt_buffer_t *in,
-                             _mongocrypt_buffer_t *out,
-                             uint32_t *bytes_written,
-                             mongocrypt_status_t *status)
+_crypto_aes_256_cbc_decrypt (_mongocrypt_crypto_t *crypto, aes_256_args_t args)
 {
-   if (enc_key->len != MONGOCRYPT_ENC_KEY_LEN) {
+   mongocrypt_status_t *status = args.status;
+   if (args.key->len != MONGOCRYPT_ENC_KEY_LEN) {
       CLIENT_ERR ("invalid encryption key length");
       return false;
    }
@@ -92,22 +81,21 @@ _crypto_aes_256_cbc_decrypt (_mongocrypt_crypto_t *crypto,
       mongocrypt_binary_t enc_key_bin, iv_bin, out_bin, in_bin;
       bool ret;
 
-      _mongocrypt_buffer_to_binary (enc_key, &enc_key_bin);
-      _mongocrypt_buffer_to_binary (iv, &iv_bin);
-      _mongocrypt_buffer_to_binary (out, &out_bin);
-      _mongocrypt_buffer_to_binary (in, &in_bin);
+      _mongocrypt_buffer_to_binary (args.key, &enc_key_bin);
+      _mongocrypt_buffer_to_binary (args.iv, &iv_bin);
+      _mongocrypt_buffer_to_binary (args.out, &out_bin);
+      _mongocrypt_buffer_to_binary (args.in, &in_bin);
 
       ret = crypto->aes_256_cbc_decrypt (crypto->ctx,
                                          &enc_key_bin,
                                          &iv_bin,
                                          &in_bin,
                                          &out_bin,
-                                         bytes_written,
+                                         args.bytes_written,
                                          status);
       return ret;
    }
-   return _native_crypto_aes_256_cbc_decrypt (
-      enc_key, iv, in, out, bytes_written, status);
+   return _native_crypto_aes_256_cbc_decrypt (args);
 }
 
 
@@ -331,13 +319,14 @@ _encrypt_step (_mongocrypt_crypto_t *crypto,
       goto done;
    }
 
-   if (!_crypto_aes_256_cbc_encrypt (crypto,
-                                     enc_key,
-                                     iv,
-                                     &to_encrypt,
-                                     ciphertext,
-                                     bytes_written,
-                                     status)) {
+   if (!_crypto_aes_256_cbc_encrypt (
+          crypto,
+          (aes_256_args_t){.key = enc_key,
+                           .iv = iv,
+                           .in = &to_encrypt,
+                           .out = ciphertext,
+                           .bytes_written = bytes_written,
+                           .status = status})) {
       goto done;
    }
 
@@ -638,7 +627,13 @@ _decrypt_step (_mongocrypt_crypto_t *crypto,
    }
 
    if (!_crypto_aes_256_cbc_decrypt (
-          crypto, iv, enc_key, ciphertext, plaintext, bytes_written, status)) {
+          crypto,
+          (aes_256_args_t){.iv = iv,
+                           .key = enc_key,
+                           .in = ciphertext,
+                           .out = plaintext,
+                           .bytes_written = bytes_written,
+                           .status = status})) {
       return false;
    }
 
@@ -994,3 +989,30 @@ _mongocrypt_unwrap_key (_mongocrypt_crypto_t *crypto,
    }
    return true;
 }
+
+bool
+_mongocrypt_hmac_sha_256 (_mongocrypt_crypto_t *crypto,
+                          const _mongocrypt_buffer_t *key,
+                          const _mongocrypt_buffer_t *in,
+                          _mongocrypt_buffer_t *out,
+                          mongocrypt_status_t *status)
+{
+   if (key->len != MONGOCRYPT_MAC_KEY_LEN) {
+      CLIENT_ERR ("invalid hmac_sha_256 key length. Got %" PRIu32
+                  ", expected: %" PRIu32,
+                  key->len,
+                  MONGOCRYPT_MAC_KEY_LEN);
+      return false;
+   }
+
+   if (crypto->hooks_enabled) {
+      mongocrypt_binary_t key_bin, out_bin, in_bin;
+      _mongocrypt_buffer_to_binary (key, &key_bin);
+      _mongocrypt_buffer_to_binary (out, &out_bin);
+      _mongocrypt_buffer_to_binary (in, &in_bin);
+
+      return crypto->hmac_sha_256 (
+         crypto->ctx, &key_bin, &in_bin, &out_bin, status);
+   }
+   return _native_crypto_hmac_sha_256 (key, in, out, status);
+}
