@@ -172,6 +172,80 @@ mongocrypt_ctx_setopt_key_alt_name (mongocrypt_ctx_t *ctx,
 }
 
 
+bool
+mongocrypt_ctx_setopt_key_material (mongocrypt_ctx_t *ctx,
+                                    mongocrypt_binary_t *key_material)
+{
+   bson_t as_bson;
+   bson_iter_t iter;
+   const char *key;
+   _mongocrypt_buffer_t buffer;
+
+   if (!ctx) {
+      return false;
+   }
+
+   if (ctx->initialized) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "cannot set options after init");
+   }
+
+   if (ctx->opts.key_material.owned) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "keyMaterial already set");
+   }
+
+   if (ctx->state == MONGOCRYPT_CTX_ERROR) {
+      return false;
+   }
+
+   if (!key_material || !key_material->data) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "option must be non-NULL");
+   }
+
+   if (!_mongocrypt_binary_to_bson (key_material, &as_bson)) {
+      return _mongocrypt_ctx_fail_w_msg (ctx,
+                                         "invalid keyMaterial bson object");
+   }
+
+   /* TODO: use _mongocrypt_parse_required_binary once MONGOCRYPT-380 is
+    * resolved.*/
+   if (!bson_iter_init (&iter, &as_bson) || !bson_iter_next (&iter)) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "invalid bson");
+   }
+
+   key = bson_iter_key (&iter);
+   BSON_ASSERT (key);
+   if (0 != strcmp (key, "keyMaterial")) {
+      return _mongocrypt_ctx_fail_w_msg (
+         ctx, "keyMaterial must have field 'keyMaterial'");
+   }
+
+   if (!_mongocrypt_buffer_from_binary_iter (&buffer, &iter)) {
+      return _mongocrypt_ctx_fail_w_msg (ctx,
+                                         "keyMaterial must be binary data");
+   }
+
+   if (buffer.len != MONGOCRYPT_KEY_LEN) {
+      _mongocrypt_set_error (
+         ctx->status,
+         MONGOCRYPT_STATUS_ERROR_CLIENT,
+         MONGOCRYPT_GENERIC_ERROR_CODE,
+         "keyMaterial should have length %d, but has length %" PRIu32,
+         MONGOCRYPT_KEY_LEN,
+         buffer.len);
+      return _mongocrypt_ctx_fail (ctx);
+   }
+
+   _mongocrypt_buffer_steal (&ctx->opts.key_material, &buffer);
+
+   if (bson_iter_next (&iter)) {
+      return _mongocrypt_ctx_fail_w_msg (
+         ctx, "unrecognized field, only keyMaterial expected");
+   }
+
+   return true;
+}
+
+
 bool
 mongocrypt_ctx_setopt_algorithm (mongocrypt_ctx_t *ctx,
                                  const char *algorithm,
@@ -292,7 +366,10 @@ _mongo_feed_keys (mongocrypt_ctx_t *ctx, mongocrypt_binary_t *in)
    _mongocrypt_buffer_t buf;
 
    _mongocrypt_buffer_from_binary (&buf, in);
-   if (!_mongocrypt_key_broker_add_doc (&ctx->kb, &buf)) {
+   if (!_mongocrypt_key_broker_add_doc (
+         &ctx->kb,
+         _mongocrypt_ctx_kms_providers(ctx),
+         &buf)) {
       BSON_ASSERT (!_mongocrypt_key_broker_status (&ctx->kb, ctx->status));
       return _mongocrypt_ctx_fail (ctx);
    }
@@ -317,7 +394,9 @@ _next_kms_ctx (mongocrypt_ctx_t *ctx)
 static bool
 _kms_done (mongocrypt_ctx_t *ctx)
 {
-   if (!_mongocrypt_key_broker_kms_done (&ctx->kb)) {
+   _mongocrypt_opts_kms_providers_t* kms_providers =
+      _mongocrypt_ctx_kms_providers(ctx);
+   if (!_mongocrypt_key_broker_kms_done (&ctx->kb, kms_providers)) {
       BSON_ASSERT (!_mongocrypt_key_broker_status (&ctx->kb, ctx->status));
       return _mongocrypt_ctx_fail (ctx);
    }
@@ -464,6 +543,62 @@ mongocrypt_ctx_next_kms_ctx (mongocrypt_ctx_t *ctx)
 }
 
 
+bool
+mongocrypt_ctx_provide_kms_providers (
+   mongocrypt_ctx_t *ctx,
+   mongocrypt_binary_t *kms_providers_definition)
+{
+   if (!ctx) {
+      return false;
+   }
+
+   if (!ctx->initialized) {
+      _mongocrypt_ctx_fail_w_msg (ctx, "ctx NULL or uninitialized");
+      return false;
+   }
+
+   if (ctx->state != MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS) {
+      _mongocrypt_ctx_fail_w_msg (ctx, "wrong state");
+      return false;
+   }
+
+   if (!_mongocrypt_parse_kms_providers(
+          kms_providers_definition,
+          &ctx->per_ctx_kms_providers,
+          ctx->status,
+          &ctx->crypt->log)) {
+      return false;
+   }
+
+   if (!_mongocrypt_opts_kms_providers_validate(
+          &ctx->per_ctx_kms_providers,
+          ctx->status)) {
+      /* Remove the parsed KMS providers if they are invalid */
+      _mongocrypt_opts_kms_providers_cleanup(&ctx->per_ctx_kms_providers);
+      memset(&ctx->per_ctx_kms_providers,
+             0,
+             sizeof(ctx->per_ctx_kms_providers));
+      return false;
+   }
+
+   memcpy(
+      &ctx->kms_providers,
+      &ctx->crypt->opts.kms_providers,
+      sizeof(_mongocrypt_opts_kms_providers_t));
+   _mongocrypt_opts_merge_kms_providers(
+      &ctx->kms_providers,
+      &ctx->per_ctx_kms_providers);
+
+   ctx->state = ctx->kb.state == KB_ADDING_DOCS ?
+      MONGOCRYPT_CTX_NEED_MONGO_KEYS :
+      MONGOCRYPT_CTX_NEED_KMS;
+   if (ctx->vtable.after_kms_credentials_provided) {
+      return ctx->vtable.after_kms_credentials_provided (ctx);
+   }
+   return true;
+}
+
+
 bool
 mongocrypt_ctx_kms_done (mongocrypt_ctx_t *ctx)
 {
@@ -544,9 +679,11 @@ mongocrypt_ctx_destroy (mongocrypt_ctx_t *ctx)
       ctx->vtable.cleanup (ctx);
    }
 
+   _mongocrypt_opts_kms_providers_cleanup(&ctx->per_ctx_kms_providers);
    _mongocrypt_kek_cleanup (&ctx->opts.kek);
    mongocrypt_status_destroy (ctx->status);
    _mongocrypt_key_broker_cleanup (&ctx->kb);
+   _mongocrypt_buffer_cleanup (&ctx->opts.key_material);
    _mongocrypt_key_alt_name_destroy_all (ctx->opts.key_alt_names);
    _mongocrypt_buffer_cleanup (&ctx->opts.key_id);
    bson_free (ctx);
@@ -687,7 +824,9 @@ _mongocrypt_ctx_init (mongocrypt_ctx_t *ctx,
       if (!ctx->opts.kek.kms_provider) {
          return _mongocrypt_ctx_fail_w_msg (ctx, "master key required");
       }
-      if (!(ctx->opts.kek.kms_provider & ctx->crypt->opts.kms_providers)) {
+      if (!ctx->crypt->opts.use_need_kms_credentials_state &&
+          !(ctx->opts.kek.kms_provider &
+            _mongocrypt_ctx_kms_providers(ctx)->configured_providers)) {
          return _mongocrypt_ctx_fail_w_msg (
             ctx, "requested kms provider not configured");
       }
@@ -733,6 +872,11 @@ _mongocrypt_ctx_init (mongocrypt_ctx_t *ctx,
       }
    }
 
+   if (opts_spec->key_material == OPT_PROHIBITED &&
+       ctx->opts.key_material.owned) {
+      return _mongocrypt_ctx_fail_w_msg (ctx, "key material prohibited");
+   }
+
    if (opts_spec->algorithm == OPT_REQUIRED &&
        ctx->opts.algorithm == MONGOCRYPT_ENCRYPTION_ALGORITHM_NONE) {
       return _mongocrypt_ctx_fail_w_msg (ctx, "algorithm required");
@@ -770,13 +914,18 @@ _mongocrypt_ctx_state_from_key_broker (mongocrypt_ctx_t *ctx)
       ret = false;
       break;
    case KB_ADDING_DOCS:
-      /* Require key documents from driver. */
-      new_state = MONGOCRYPT_CTX_NEED_MONGO_KEYS;
+      /* Encrypted keys need KMS, which need to be provided before
+       * adding docs. */
+      if (_mongocrypt_needs_credentials (ctx->crypt)) {
+         new_state = MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS;
+      } else {
+         /* Require key documents from driver. */
+         new_state = MONGOCRYPT_CTX_NEED_MONGO_KEYS;
+      }
       ret = true;
       break;
    case KB_AUTHENTICATING:
    case KB_DECRYPTING_KEY_MATERIAL:
-      /* Encrypted keys need KMS. */
       new_state = MONGOCRYPT_CTX_NEED_KMS;
       ret = true;
       break;
@@ -885,3 +1034,10 @@ mongocrypt_ctx_setopt_key_encryption_key (mongocrypt_ctx_t *ctx,
 
    return true;
 }
+
+_mongocrypt_opts_kms_providers_t *
+_mongocrypt_ctx_kms_providers(mongocrypt_ctx_t *ctx) {
+   return ctx->kms_providers.configured_providers ?
+      &ctx->kms_providers :
+      &ctx->crypt->opts.kms_providers;
+}
