@@ -334,6 +334,10 @@ _mongocrypt_tester_run_ctx_to (_mongocrypt_tester_t *tester,
          ASSERT_OR_PRINT (res, status);
          BSON_ASSERT (mongocrypt_ctx_mongo_done (ctx));
          break;
+      case MONGOCRYPT_CTX_NEED_KMS_CREDENTIALS:
+         bin = TEST_BSON ("{}");
+         mongocrypt_ctx_provide_kms_providers (ctx, bin);
+         break;
       case MONGOCRYPT_CTX_NEED_KMS:
          kms = mongocrypt_ctx_next_kms_ctx (ctx);
          while (kms) {
@@ -654,6 +658,12 @@ _test_setopt_invalid_kms_providers (_mongocrypt_tester_t *tester)
    mongocrypt_status_destroy (status);
    mongocrypt_ctx_destroy (ctx);
    mongocrypt_destroy (crypt);
+
+   crypt = mongocrypt_new ();
+   mongocrypt_setopt_use_need_kms_credentials_state (crypt);
+   ASSERT_OK (mongocrypt_setopt_kms_providers (crypt, TEST_BSON ("{}")), crypt);
+   ASSERT_FAILS (mongocrypt_init (crypt), crypt, "no kms provider set");
+   mongocrypt_destroy (crypt);
 }
 
 typedef struct {
@@ -682,13 +692,11 @@ _test_setopt_kms_providers (_mongocrypt_tester_t *tester)
        "Invalid endpoint"},
       {"{'azure': {'tenantId': '', 'clientSecret': '' }}", "clientId"},
       {"{'aws': {'accessKeyId': 'abc', 'secretAccessKey': 'def'}}", NULL},
-      {"{'aws': {}}", "expected UTF-8 aws.accessKeyId"},
       {"{'local': {'key': {'$binary': {'base64': '" EXAMPLE_LOCAL_MATERIAL
        "', 'subType': '00'}} }}",
        NULL},
       {"{'local': {'key': '" EXAMPLE_LOCAL_MATERIAL "' }}", NULL},
       {"{'local': {'key': 'invalid base64' }}", "unable to parse base64"},
-      {"{'local': {}}", "expected UTF-8 or binary local.key"},
       /* either base64 string or binary is acceptable for privateKey */
       {"{'gcp': {'endpoint': 'oauth2.googleapis.com', 'email': 'test', "
        "'privateKey': 'AAAA' }}"},
@@ -716,12 +724,18 @@ _test_setopt_kms_providers (_mongocrypt_tester_t *tester)
       {"{'local': {'key': 'AAAA'}}", "local key must be 96 bytes"},
       /* KMIP test cases. */
       {"{'kmip': {'endpoint': '127.0.0.1:5696' }}", NULL},
-      {"{'kmip': {}}", "expected endpoint kmip.endpoint"},
       /* localhost is a valid endpoint for KMIP.
        * Unlike Azure, GCP, and AWS, applications run their own KMIP servers. */
       {"{'kmip': {'endpoint': 'localhost' }}", NULL},
       {"{'kmip': {'endpoint': '127.0.0.1:5696', 'extra': 'invalid' }}",
-       "Unexpected field: 'extra'"}};
+       "Unexpected field: 'extra'"},
+       /* Empty documents are OK for on-demand KMS credentials */
+      {"{'aws': {}}", NULL},
+      {"{'azure': {}}", NULL},
+      {"{'local': {}}", NULL},
+      {"{'gcp': {}}", NULL},
+      {"{'kmip': {}}", NULL}
+       };
 
    for (i = 0; i < sizeof (tests) / sizeof (tests[0]); i++) {
       mongocrypt_t *crypt;
@@ -788,6 +802,8 @@ main (int argc, char **argv)
    _mongocrypt_tester_install_kek (&tester);
    _mongocrypt_tester_install_cache_oauth (&tester);
    _mongocrypt_tester_install_kms_ctx (&tester);
+   _mongocrypt_tester_install_csfle_lib (&tester);
+   _mongocrypt_tester_install_dll (&tester);
 
 
    printf ("Running tests...\n");
