@@ -173,6 +173,12 @@ _mongocrypt_tester_install_kms_responses (_mongocrypt_tester_t *tester);
 void
 _mongocrypt_tester_install_status (_mongocrypt_tester_t *tester);
 
+void
+_mongocrypt_tester_install_csfle_lib (_mongocrypt_tester_t *tester);
+
+void
+_mongocrypt_tester_install_dll (_mongocrypt_tester_t *tester);
+
 void
 _mongocrypt_tester_install_endpoint (_mongocrypt_tester_t *tester);
 
